@@ -29,12 +29,16 @@ instance ToASCII EXPRESSION where
   toASCII EX_TERMINATION {..} = EX_TERMINATION T
   toASCII EX_FORMATION {..} = EX_FORMATION LSB' eol tab (toASCII binding) eol' tab' RSB'
   toASCII EX_DISPATCH {..} = EX_DISPATCH (toASCII expr) (toASCII attr)
-  toASCII EX_APPLICATION {..} = EX_APPLICATION (toASCII expr) eol tab (toASCII bindings) eol' tab'
-  toASCII EX_APPLICATION' {..} = EX_APPLICATION' (toASCII expr) eol tab (toASCII args) eol' tab'
+  toASCII EX_APPLICATION {..} = EX_APPLICATION (toASCII expr) eol tab (toASCII tau) eol' tab' indent
+  toASCII EX_APPLICATION_TAUS {..} = EX_APPLICATION_TAUS (toASCII expr) eol tab (toASCII taus) eol' tab' indent
+  toASCII EX_APPLICATION_EXPRS {..} = EX_APPLICATION_EXPRS (toASCII expr) eol tab (toASCII args) eol' tab' indent
   toASCII EX_META {..} = EX_META (MT_EXPRESSION' (rest meta))
   toASCII EX_META_TAIL {..} = EX_META_TAIL (toASCII expr) (MT_TAIL (rest meta))
   toASCII expr = expr
 
+instance ToASCII APP_BINDING where
+  toASCII APP_BINDING {..} = APP_BINDING (toASCII pair)
+
 instance ToASCII BINDING where
   toASCII BI_PAIR {..} = BI_PAIR (toASCII pair) (toASCII bindings) tab
   toASCII BI_META {..} = BI_META (MT_BINDING' (rest meta)) (toASCII bindings) tab
@@ -54,6 +58,7 @@ instance ToASCII APP_ARGS where
 
 instance ToASCII PAIR where
   toASCII PA_TAU {..} = PA_TAU (toASCII attr) ARROW' (toASCII expr)
+  toASCII PA_FORMATION{..} = PA_FORMATION (toASCII attr) (map toASCII voids) ARROW' (toASCII expr)
   toASCII PA_VOID {..} = PA_VOID (toASCII attr) ARROW' QUESTION
   toASCII PA_LAMBDA {..} = PA_LAMBDA' func
   toASCII PA_DELTA {..} = PA_DELTA' bytes
