@@ -40,28 +40,18 @@ rewrittensToLatex rewrittens ctx@LatexContext {..} =
   let equation = phiquation ctx
    in concat
         [ printf "\\begin{%s}\n" equation,
-          case label of
-            Just lbl -> printf "\\label{%s}\n" lbl
-            _ -> "",
-          case expression of
-            Just expr -> printf "\\phiExpression{%s} " expr
-            _ -> "",
+          maybe "" (printf "\\label{%s}\n") label,
+          maybe "" (printf "\\phiExpression{%s} ") expression,
           intercalate
             "\n  \\leadsto "
             ( map
-                ( \Rewritten {..} ->
+                ( \(program, maybeName) ->
                     let prog = renderToLatex program ctx
-                        unknown = "unknown"
-                        maybeTo = case maybeRule of
-                          Just (Y.Rule {..}) -> Just (map toLower (fromMaybe unknown name))
-                          _ -> Nothing
-                     in case maybeTo of
-                          Just name -> printf "%s \\leadsto_{\\nameref{r:%s}}" prog name
-                          _ -> prog
+                     in maybe prog (printf "%s \\leadsto_{\\nameref{r:%s}}" prog) maybeName
                 )
                 rewrittens
             ),
-          printf "\n\\end{%s}" equation
+          printf ".\n\\end{%s}" equation
         ]
 
 programToLaTeX :: Program -> LatexContext -> String
@@ -90,8 +80,9 @@ instance ToLaTeX PROGRAM where
 instance ToLaTeX EXPRESSION where
   toLaTeX EX_ATTR {..} = EX_ATTR (toLaTeX attr)
   toLaTeX EX_FORMATION {..} = EX_FORMATION lsb eol tab (toLaTeX binding) eol' tab' rsb
-  toLaTeX EX_APPLICATION {..} = EX_APPLICATION (toLaTeX expr) eol tab (toLaTeX bindings) eol' tab'
-  toLaTeX EX_APPLICATION' {..} = EX_APPLICATION' (toLaTeX expr) eol tab (toLaTeX args) eol' tab'
+  toLaTeX EX_APPLICATION {..} = EX_APPLICATION (toLaTeX expr) eol tab (toLaTeX tau) eol' tab' indent
+  toLaTeX EX_APPLICATION_TAUS {..} = EX_APPLICATION_TAUS (toLaTeX expr) eol tab (toLaTeX taus) eol' tab' indent
+  toLaTeX EX_APPLICATION_EXPRS {..} = EX_APPLICATION_EXPRS (toLaTeX expr) eol tab (toLaTeX args) eol' tab' indent
   toLaTeX EX_DISPATCH {..} = EX_DISPATCH (toLaTeX expr) (toLaTeX attr)
   toLaTeX expr = expr
 
@@ -99,6 +90,9 @@ instance ToLaTeX ATTRIBUTE where
   toLaTeX AT_LABEL {..} = AT_LABEL (piped (toLaTeX label))
   toLaTeX attr = attr
 
+instance ToLaTeX APP_BINDING where
+  toLaTeX APP_BINDING {..} = APP_BINDING (toLaTeX pair)
+
 instance ToLaTeX BINDING where
   toLaTeX BI_PAIR {..} = BI_PAIR (toLaTeX pair) (toLaTeX bindings) tab
   toLaTeX bd = bd
@@ -113,6 +107,7 @@ instance ToLaTeX PAIR where
   toLaTeX PA_LAMBDA' {..} = PA_LAMBDA' (piped (toLaTeX func))
   toLaTeX PA_VOID {..} = PA_VOID (toLaTeX attr) arrow void
   toLaTeX PA_TAU {..} = PA_TAU (toLaTeX attr) arrow (toLaTeX expr)
+  toLaTeX PA_FORMATION {..} = PA_FORMATION (toLaTeX attr) (map toLaTeX voids) arrow (toLaTeX expr)
   toLaTeX pair = pair
 
 instance ToLaTeX APP_ARG where
