@@ -1,3 +1,5 @@
+{-# LANGUAGE RecordWildCards #-}
+
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
 -- SPDX-License-Identifier: MIT
 
@@ -6,32 +8,42 @@ module Logger
     logInfo,
     logWarning,
     logError,
-    setLogLevel,
+    setLogConfig,
     LogLevel (DEBUG, INFO, WARNING, ERROR, NONE),
   )
 where
 
 import Control.Monad (when)
 import Data.IORef (IORef, newIORef, readIORef, writeIORef)
+import qualified Data.List as DL
 import GHC.IO (unsafePerformIO)
 import System.IO
 
 data LogLevel = DEBUG | INFO | WARNING | ERROR | NONE
   deriving (Show, Ord, Eq, Bounded, Enum, Read)
 
-newtype Logger = Logger {level :: LogLevel}
+data Logger = Logger {level :: LogLevel, lines :: Int}
 
 logger :: IORef Logger
 {-# NOINLINE logger #-}
-logger = unsafePerformIO (newIORef (Logger INFO))
+logger = unsafePerformIO (newIORef (Logger INFO 25))
 
-setLogLevel :: LogLevel -> IO ()
-setLogLevel lvl = writeIORef logger (Logger lvl)
+setLogConfig :: LogLevel -> Integer -> IO ()
+setLogConfig lvl lines = writeIORef logger (Logger lvl (fromIntegral lines))
 
 logMessage :: LogLevel -> String -> IO ()
 logMessage lvl message = do
-  log <- readIORef logger
-  when (lvl >= level log) $ hPutStrLn stderr ("[" ++ show lvl ++ "]: " ++ message)
+  Logger {..} <- readIORef logger
+  when
+    (lvl >= level && lines /= 0)
+    ( let lines' = DL.lines message
+          toPrint = take lines lines'
+          msg
+            | lines == -1 = [message]
+            | length lines' > lines = toPrint ++ ["---| log is limited by --log-lines=" ++ show lines ++ " option |---"]
+            | otherwise = toPrint
+       in hPutStrLn stderr ("[" ++ show lvl ++ "]: " ++ DL.intercalate "\n" msg)
+    )
 
 logDebug, logInfo, logWarning, logError :: String -> IO ()
 logDebug = logMessage DEBUG
