@@ -14,6 +14,7 @@ module Printer
     printBytes,
     printExtraArg,
     printSubsts,
+    printSubsts',
     PrintConfig (..),
     logPrintConfig,
   )
@@ -52,7 +53,7 @@ printExpression :: Expression -> String
 printExpression expr = printExpression' expr defaultPrintConfig
 
 printAttribute' :: Attribute -> Encoding -> String
-printAttribute' attr encoding = render (withEncoding encoding (toCST attr 0 :: ATTRIBUTE))
+printAttribute' attr encoding = render (withEncoding encoding (toCST attr 0 NO_EOL :: ATTRIBUTE))
 
 printAttribute :: Attribute -> String
 printAttribute attr =
@@ -66,7 +67,7 @@ printBinding :: Binding -> String
 printBinding bd = printBinding' bd defaultPrintConfig
 
 printBytes :: Bytes -> String
-printBytes bts = render (toCST bts 0 :: BYTES)
+printBytes bts = render (toCST bts 0 NO_EOL :: BYTES)
 
 printExtraArg' :: ExtraArgument -> PrintConfig -> String
 printExtraArg' (ArgAttribute attr) (_, encoding, _) = printAttribute' attr encoding
@@ -91,13 +92,13 @@ printMetaValue (MvTail tails) config = intercalate "," (map (`printTail` config)
 
 printSubst :: Subst -> PrintConfig -> String
 printSubst (Subst mp) config =
-  "  "
-    <> intercalate
-      "\n  "
-      (map (\(key, value) -> key <> " >> " <> printMetaValue value config) (Map.toList mp))
+  intercalate
+    "\n"
+    (map (\(key, value) -> key <> " >> " <> printMetaValue value config) (Map.toList mp))
 
 printSubsts' :: [Subst] -> PrintConfig -> String
-printSubsts' substs config = intercalate "\n---\n" (map (`printSubst` config) substs)
+printSubsts' [] _ = "------"
+printSubsts' substs config = intercalate "\n------\n" (map (`printSubst` config) substs)
 
 printSubsts :: [Subst] -> String
 printSubsts substs = printSubsts' substs defaultPrintConfig
