@@ -1,4 +1,5 @@
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE NamedFieldPuns #-}
 {-# LANGUAGE RecordWildCards #-}
 
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
@@ -125,6 +126,8 @@ instance Render PROGRAM where
 
 instance Render PAIR where
   render PA_TAU {..} = render attr <> render SPACE <> render arrow <> render SPACE <> render expr
+  render PA_FORMATION {voids = [], attr, arrow, expr} = render (PA_TAU attr arrow expr)
+  render PA_FORMATION {..} = render attr <> "(" <> intercalate ", " (map render voids) <> ")" <> render SPACE <> render arrow <> render SPACE <> render expr
   render PA_LAMBDA {..} = render LAMBDA <> render SPACE <> render DASHED_ARROW <> render SPACE <> render func
   render PA_LAMBDA' {..} = "L> " <> func
   render PA_VOID {..} = render attr <> render SPACE <> render arrow <> render SPACE <> render void
@@ -140,6 +143,9 @@ instance Render BINDINGS where
   render BDS_PAIR {..} = render COMMA <> render eol <> render tab <> render pair <> render bindings
   render BDS_META {..} = render COMMA <> render eol <> render tab <> render meta <> render bindings
 
+instance Render APP_BINDING where
+  render APP_BINDING {..} = render pair
+
 instance Render BINDING where
   render BI_EMPTY {..} = ""
   render BI_PAIR {..} = render pair <> render bindings
@@ -152,10 +158,6 @@ instance Render APP_ARGS where
   render AAS_EMPTY = ""
   render AAS_EXPR {..} = render COMMA <> render eol <> render tab <> render expr <> render args
 
--- @todo #163:30min Introduce node for formation with inlined voids.
---  We need to be able to print formation with inlined void attributes:
---  x(a, b) -> [[ y -> 1 ]] => x -> [[ a -> ?, b -> ?, y -> 1 ]]
---  Don't forget to extend toSalty instance so such sugar.
 instance Render EXPRESSION where
   render EX_GLOBAL {..} = render global
   render EX_DEF_PACKAGE {..} = render pckg
@@ -164,8 +166,9 @@ instance Render EXPRESSION where
   render EX_TERMINATION {..} = render termination
   render EX_FORMATION {..} = render lsb <> render eol <> render tab <> render binding <> render eol' <> render tab' <> render rsb
   render EX_DISPATCH {..} = render expr <> "." <> render attr
-  render EX_APPLICATION {..} = render expr <> "(" <> render eol <> render tab <> render bindings <> render eol' <> render tab' <> ")"
-  render EX_APPLICATION' {..} = render expr <> "(" <> render eol <> render tab <> render args <> render eol' <> render tab' <> ")"
+  render EX_APPLICATION {..} = render expr <> "(" <> render eol <> render tab <> render tau <> render eol' <> render tab' <> ")"
+  render EX_APPLICATION_TAUS {..} = render expr <> "(" <> render eol <> render tab <> render taus <> render eol' <> render tab' <> ")"
+  render EX_APPLICATION_EXPRS {..} = render expr <> "(" <> render eol <> render tab <> render args <> render eol' <> render tab' <> ")"
   render EX_STRING {..} = '"' : render str <> "\""
   render EX_NUMBER {..} = either show show num
   render EX_META {..} = render meta
