@@ -7,11 +7,12 @@
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
 -- SPDX-License-Identifier: MIT
 
-module Rewriter (rewrite, rewrite', RewriteContext (..), Rewritten (..)) where
+module Rewriter (rewrite, rewrite', RewriteContext (..), Rewritten) where
 
 import AST
 import Builder
 import Control.Exception (Exception, throwIO)
+import Data.Char (toLower)
 import Data.Foldable (foldlM)
 import Data.Functor ((<&>))
 import qualified Data.Map.Strict as M
@@ -33,19 +34,12 @@ import qualified Yaml as Y
 
 type RewriteState = ([Rewritten], Set.Set Program)
 
-data Rewritten = Rewritten
-  { program :: Program,
-    maybeRule :: Maybe Y.Rule
-  }
-
-instance Eq Rewritten where
-  left == right = program left == program right
+type Rewritten = (Program, Maybe String)
 
 data RewriteContext = RewriteContext
   { _maxDepth :: Integer,
     _maxCycles :: Integer,
     _depthSensitive :: Bool,
-    _sequence :: Bool,
     _buildTerm :: BuildTermFunc,
     _must :: Must,
     _saveStep :: SaveStepFunc
@@ -131,7 +125,7 @@ tryBuildAndReplaceFast (ExFormation pbds) (ExFormation rbds) substs ctx =
 tryBuildAndReplaceFast ptn res substs ctx = buildAndReplace' ptn res substs replaceProgramThrows ctx
 
 -- The function returns tuple (X, Y) where
--- - X is sequence of programs; for more details about this sequence see description of rewrite' function
+-- - X is sequence of programs;
 -- - Y is Set of unique programs after each rule application. It allows to stop the rewriting if we're getting
 --   into loop and get back to program which we've already got before
 rewrite :: RewriteState -> [Y.Rule] -> Integer -> RewriteContext -> IO RewriteState
@@ -145,7 +139,7 @@ rewrite state (rule : rest) iteration ctx@RewriteContext {..} = do
       let ruleName = fromMaybe "unknown" (Y.name rule)
           ptn = Y.pattern rule
           res = Y.result rule
-          Rewritten {..} = head _rewrittens
+          (program, _) = head _rewrittens
        in if _count - 1 == _maxDepth
             then do
               logDebug (printf "Max amount of rewriting cycles (%d) for rule '%s' has been reached, rewriting is stopped" _maxDepth ruleName)
@@ -179,24 +173,19 @@ rewrite state (rule : rest) iteration ctx@RewriteContext {..} = do
                                 (printProgram prog)
                             )
                           _saveStep prog (((iteration - 1) * _maxDepth) + _count)
-                          _rewrite (rewriteSequence prog, Set.insert prog _unique) (_count + 1)
+                          _rewrite (leadsTo prog, Set.insert prog _unique) (_count + 1)
       where
-        rewriteSequence :: Program -> [Rewritten]
-        rewriteSequence _prog
-          | _sequence =
-              let Rewritten {..} : rest = _rewrittens
-               in Rewritten _prog Nothing : Rewritten program (Just rule) : rest
-          | otherwise = [Rewritten _prog Nothing]
+        leadsTo :: Program -> [Rewritten]
+        leadsTo _prog =
+          let (program, _) : rest = _rewrittens
+           in (_prog, Nothing) : (program, Just (map toLower (fromMaybe "unknown" (Y.name rule)))) : rest
 
 -- The function accepts single program but returns sequence of programs
--- If RewriteContext has _sequence == True - all the intermediate
--- programs after each rule application are included into sequence
--- Otherwise sequence contains only one program
 rewrite' :: Program -> [Y.Rule] -> RewriteContext -> IO [Rewritten]
-rewrite' prog rules ctx = _rewrite ([Rewritten prog Nothing], Set.empty) 1 ctx <&> reverse
+rewrite' prog rules ctx = _rewrite ([(prog, Nothing)], Set.empty) 1 ctx <&> reverse
   where
     _rewrite :: RewriteState -> Integer -> RewriteContext -> IO [Rewritten]
-    _rewrite state@(rewrittens, unique) count ctx@RewriteContext{..} = do
+    _rewrite state@(rewrittens, unique) count ctx@RewriteContext {..} = do
       let cycles = _maxCycles
           must = _must
           current = count - 1
@@ -212,7 +201,9 @@ rewrite' prog rules ctx = _rewrite ([Rewritten prog Nothing], Set.empty) 1 ctx <
             else do
               logDebug (printf "Starting rewriting cycle for all rules: %d out of %d" count cycles)
               state'@(rewrittens', unique') <- rewrite state rules count ctx
-              if head rewrittens' == head rewrittens
+              let (program', _) = head rewrittens'
+                  (program, _) = head rewrittens
+              if program' == program
                 then do
                   logDebug "Rewriting is stopped since it has no effect"
                   if not (inRange must current)
