@@ -10,7 +10,7 @@ module CLISpec (spec) where
 import CLI (runCLI)
 import Control.Exception
 import Control.Monad (forM_, unless, when)
-import Data.List (isInfixOf)
+import Data.List (intercalate, isInfixOf)
 import Data.Version (showVersion)
 import GHC.IO.Handle
 import Paths_phino (version)
@@ -73,13 +73,18 @@ withTempFile pattern =
 testCLI' :: [String] -> [String] -> Either ExitCode () -> Expectation
 testCLI' args outputs exit = do
   (out, result) <- withStdout (try (runCLI args) :: IO (Either ExitCode ()))
-  forM_
-    outputs
-    ( \output ->
-        unless (output `isInfixOf` out) $
-          expectationFailure
-            ("Expected that output contains:\n" ++ output ++ "\nbut got:\n" ++ out)
-    )
+  if null outputs
+    then
+      unless (null out) $
+        expectationFailure ("Expected that output is empty, but got:\n" ++ out)
+    else
+      forM_
+        outputs
+        ( \output ->
+            unless (output `isInfixOf` out) $
+              expectationFailure
+                ("Expected that output contains:\n" ++ output ++ "\nbut got:\n" ++ out)
+        )
   result `shouldBe` exit
 
 testCLISucceeded :: [String] -> [String] -> Expectation
@@ -116,6 +121,12 @@ spec = do
             ["rewrite", "--input=latex"]
             ["The value 'latex' can't be used for '--input' option"]
 
+      it "with negative --log-lines" $
+        withStdin "" $
+          testCLIFailed
+            ["rewrite", "--log-lines=-2"]
+            ["--log-lines must be >= -1"]
+
       it "with negative --max-depth" $
         withStdin "" $
           testCLIFailed
@@ -199,6 +210,18 @@ spec = do
             ["rewrite", "--hide=Q.x(Q.y)"]
             ["[ERROR]: Invalid set of arguments: Only dispatch expression", "but given: Φ.x( Φ.y )"]
 
+      it "with many --show options" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["rewrite", "--show=Q.x.y", "--show=hello"]
+            ["The option --show can be used only once"]
+      
+      it "with wrong --show option" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["rewrite", "--show=Q.x(Q.y)"]
+            ["[ERROR]:", "Only dispatch expression started with Φ (or Q) can be used in --show"]
+
     it "saves steps to dir with --steps-dir" $ do
       let dir = "test-steps-temp"
       dirExists <- doesDirectoryExist dir
@@ -276,9 +299,7 @@ spec = do
           ["rewrite", "--output=latex", "--sweet"]
           [ "\\begin{phiquation}",
             "\\Big\\{[[\n",
-            "  |x\\char95{}o| -> QQ.|z|(\n",
-            "    |y| -> 5\n",
-            "  ),\n",
+            "  |x\\char95{}o| -> QQ.|z|( |y| -> 5 ),\n",
             "  |q\\char36{}| -> T,\n",
             "  |w| -> $,\n",
             "  ^ -> Q,\n",
@@ -303,7 +324,7 @@ spec = do
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet", "--flat", "--expression=foo"]
           [ "\\begin{phiquation}",
-            "\\phiExpression{foo} \\Big\\{[[ |x| -> 5 ]]\\Big\\}",
+            "\\phiExpression{foo} \\Big\\{[[ |x| -> 5 ]]\\Big\\}.\n",
             "\\end{phiquation}"
           ]
 
@@ -312,7 +333,7 @@ spec = do
         testCLISucceeded
           ["rewrite", "--output=latex", "--sweet", "--flat", "--label=foo"]
           [ "\\begin{phiquation}\n\\label{foo}\n",
-            "\\Big\\{[[ |x| -> 5 ]]\\Big\\}\n",
+            "\\Big\\{[[ |x| -> 5 ]]\\Big\\}.\n",
             "\\end{phiquation}"
           ]
 
@@ -377,7 +398,7 @@ spec = do
               [ "\\begin{phiquation}",
                 "\\Big\\{[[ |x| -> \"foo\" ]]\\Big\\} \\leadsto_{\\nameref{r:first}}",
                 "  \\leadsto \\Big\\{Q.|x|( |y| -> \"foo\" )\\Big\\} \\leadsto_{\\nameref{r:second}}",
-                "  \\leadsto \\Big\\{[[ |x| -> \"foo\" ]]\\Big\\}",
+                "  \\leadsto \\Big\\{[[ |x| -> \"foo\" ]]\\Big\\}.",
                 "\\end{phiquation}"
               ]
           ]
@@ -493,20 +514,119 @@ spec = do
           ["rewrite", "--sweet", "--flat", "--hide=Q.org.eolang", "--hide=Q.org.yegor256"]
           ["{⟦ org ↦ ⟦⟧, x ↦ 42 ⟧}"]
 
+    it "shows and hides" $
+      withStdin "{[[ org -> [[ eolang -> Q.x, yegor256 -> Q.y ]], x -> 42 ]]}" $
+        testCLISucceeded
+          ["rewrite", "--sweet", "--flat", "--show=Q.org", "--hide=Q.org.eolang"]
+          ["{⟦ org ↦ ⟦ yegor256 ↦ Φ.y ⟧ ⟧}"]
+
     it "prints in line with --flat" $
       withStdin "Q -> [[ x -> 5, y -> \"hey\", z -> [[ w -> [[ ]] ]] ]]" $
         testCLISucceeded
           ["rewrite", "--sweet", "--flat"]
           ["{⟦ x ↦ 5, y ↦ \"hey\", z ↦ ⟦ w ↦ ⟦⟧ ⟧ ⟧}"]
 
+    it "removes unnecessary rho bindings in primitive applications" $
+      withStdin
+        ( unlines
+            [ "{[[",
+              "  z -> [[ x -> [[ t -> 42 ]].t ]].x,",
+              "  org -> [[ eolang -> [[ bytes -> [[ data -> ? ]], number -> [[ as-bytes -> ? ]] ]] ]]",
+              "]]}"
+            ]
+        )
+        ( testCLISucceeded
+            ["rewrite", "--sweet", "--normalize", "--flat"]
+            ["{⟦ z ↦ 42, org ↦ ⟦ eolang ↦ ⟦ bytes(data) ↦ ⟦⟧, number(as-bytes) ↦ ⟦⟧ ⟧ ⟧ ⟧}"]
+        )
+
+    it "reduces log message" $
+      withStdin "{[[ x -> [[ y -> ? ]](y -> 5) ]]}" $
+        testCLISucceeded
+          ["rewrite", "--log-level=debug", "--log-lines=4", "--normalize"]
+          [ intercalate
+              "\n"
+              [ "[DEBUG]: Applied 'COPY' (28 nodes -> 25 nodes)",
+                "{⟦",
+                "  x ↦ ⟦",
+                "    y ↦ 5",
+                "---| log is limited by --log-lines=4 option |---"
+              ]
+          ]
+
+    it "canonizes program" $
+      withStdin "{[[ x -> [[ y -> [[ L> Func ]].q, z -> Q.x(a -> [[ w -> [[ L> Atom ]], L> Hello ]]) ]], L> Package ]]}" $
+        testCLISucceeded
+          ["rewrite", "--canonize", "--sweet", "--flat"]
+          ["{⟦ x ↦ ⟦ y ↦ ⟦ λ ⤍ F1 ⟧.q, z ↦ Φ.x( a ↦ ⟦ w ↦ ⟦ λ ⤍ F2 ⟧, λ ⤍ F3 ⟧ ) ⟧, λ ⤍ F4 ⟧}"]
+
   describe "dataize" $ do
     it "dataizes simple program" $
       withStdin "Q -> [[ D> 01- ]]" $
         testCLISucceeded ["dataize"] ["01-"]
 
-    it "fails to dataize" $
+    it "dataizes to dead" $
       withStdin "Q -> [[ ]]" $
-        testCLIFailed ["dataize"] ["[ERROR]: Could not dataize given program"]
+        testCLISucceeded ["dataize"] ["⊥"]
+
+    it "dataizes with --sequence" $
+      withStdin "{[[ @ -> [[ x -> [[ D> 01-, y -> ? ]](y -> [[ ]]) ]].x ]]}" $
+        testCLISucceeded
+          ["dataize", "--sequence", "--output=latex", "--flat", "--sweet"]
+          [ intercalate
+              "\n"
+              [ "\\begin{phiquation}",
+                "\\Big\\{[[ @ -> [[ |x| -> [[ D> 01-, |y| -> ? ]]( |y| -> [[]] ) ]].|x| ]]\\Big\\} \\leadsto_{\\nameref{r:contextualize}}",
+                "  \\leadsto \\Big\\{[[ |x| -> [[ D> 01-, |y| -> ? ]]( |y| -> [[]] ) ]].|x|\\Big\\} \\leadsto_{\\nameref{r:copy}}",
+                "  \\leadsto \\Big\\{[[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]].|x|\\Big\\} \\leadsto_{\\nameref{r:dot}}",
+                "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]] ]]( ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] )\\Big\\} \\leadsto_{\\nameref{r:copy}}",
+                "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]], ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] ]]\\Big\\} \\leadsto_{\\nameref{r:Mprim}}",
+                "  \\leadsto \\Big\\{[[ D> 01-, |y| -> [[]], ^ -> [[ |x| -> [[ D> 01-, |y| -> [[]] ]] ]] ]]\\Big\\}.",
+                "\\end{phiquation}",
+                "01-"
+              ]
+          ]
+
+    it "does not print bytes with --quiet" $
+      withStdin "Q -> [[ D> 01- ]]" $
+        testCLISucceeded ["dataize", "--quiet"] []
+
+    describe "fails" $ do
+      it "with --output != latex and --nonumber" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["dataize", "--nonumber", "--output=xmir"]
+            ["The --nonumber option can stay together with --output=latex only"]
+
+      it "with --omit-listing and --output != xmir" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["dataize", "--omit-listing", "--output=phi"]
+            ["--omit-listing"]
+
+      it "with --omit-comments and --output != xmir" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["dataize", "--omit-comments", "--output=phi"]
+            ["--omit-comments"]
+
+      it "with --expression and --output != latex" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["dataize", "--expression=foo", "--output=phi"]
+            ["--expression option can stay together with --output=latex only"]
+
+      it "with --label and --output != latex" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["dataize", "--label=foo", "--output=phi"]
+            ["--label option can stay together with --output=latex only"]
+
+      it "with wrong --hide option" $
+        withStdin "{[[]]}" $
+          testCLIFailed
+            ["dataize", "--hide=Q.x(Q.y)"]
+            ["[ERROR]: Invalid set of arguments: Only dispatch expression", "but given: Φ.x( Φ.y )"]
 
   describe "explain" $ do
     it "explains single rule" $
@@ -556,15 +676,9 @@ spec = do
             [ "{⟦",
               "  org ↦ ⟦",
               "    eolang ↦ ⟦",
-              "      number ↦ ⟦",
-              "        φ ↦ ∅",
-              "      ⟧,",
-              "      bytes ↦ ⟦",
-              "        data ↦ ∅",
-              "      ⟧,",
-              "      string ↦ ⟦",
-              "        φ ↦ ∅",
-              "      ⟧,",
+              "      number(φ) ↦ ⟦⟧,",
+              "      bytes(data) ↦ ⟦⟧,",
+              "      string(φ) ↦ ⟦⟧,",
               "      λ ⤍ Package",
               "    ⟧,",
               "    λ ⤍ Package",
@@ -587,3 +701,40 @@ spec = do
       testCLIFailed
         ["merge"]
         ["At least one input file must be specified for 'merge' command"]
+
+  describe "match" $ do
+    it "takes from stdin" $
+      withStdin "{[[]]}" $
+        testCLISucceeded ["match"] ["[INFO]"]
+
+    it "takes from file" $
+      testCLISucceeded ["match", "test-resources/cli/foo.phi"] ["[INFO]"]
+
+    it "does not print substitutions without pattern" $
+      withStdin "{[[]]}" $
+        testCLISucceeded ["match"] ["[INFO]: The --pattern is not provided, no substitutions are built"]
+
+    it "prints one substitution" $
+      withStdin "{[[ x -> Q.x ]]}" $
+        testCLISucceeded ["match", "--pattern=Q.!a"] ["a >> x"]
+
+    it "prints many substitutions" $
+      withStdin "{[[ x -> Q.x, y -> Q.y ]]}" $
+        testCLISucceeded ["match", "--pattern=Q.!a"] ["a >> x\n------\na >> y"]
+
+    it "builds substitutions with conditions" $
+      withStdin "{[[ x -> Q.y ]].x}" $
+        testCLISucceeded
+          ["match", "--pattern=[[ !a -> Q.y, !B ]].!a", "--when=and(not(alpha(!a)),eq(length(!B),1))"]
+          ["B >> ⟦ ρ ↦ ∅ ⟧\na >> x"]
+
+    it "builds with condition from file" $
+      testCLISucceeded
+        ["match", "--pattern=[[ !B ]]", "--when=eq(length(!B),2)", "test-resources/cli/foo.phi"]
+        ["B >> ⟦\n  foo ↦ Φ.org.eolang.x,\n  ρ ↦ ∅\n⟧"]
+
+    it "fails on parsing --when condition" $
+      withStdin "{[[]]}" $
+        testCLIFailed
+          ["match", "--pattern=[[!B]]", "--when=hello"]
+          ["[ERROR]: Couldn't parse given condition"]
